"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const minimatch = require("minimatch");
const R = require("ramda");
function filterGraphByEntryDirectory(pkgGraph, entryDirectory) {
    if (!pkgGraph[entryDirectory])
        return {};
    const walkedDependencies = new Set();
    const graph = pkgGraphToGraph(pkgGraph);
    pickSubgraph(graph, [entryDirectory], walkedDependencies);
    return R.pick(Array.from(walkedDependencies), pkgGraph);
}
exports.filterGraphByEntryDirectory = filterGraphByEntryDirectory;
function filterGraph(pkgGraph, filters) {
    const cherryPickedPackages = [];
    const walkedDependencies = new Set();
    const walkedDependents = new Set();
    const graph = pkgGraphToGraph(pkgGraph);
    let reversedGraph;
    for (const filter of filters) {
        if (filter.endsWith('...')) {
            const rootPackagesFilter = filter.substring(0, filter.length - 3);
            const rootPackages = matchPackages(pkgGraph, rootPackagesFilter);
            pickSubgraph(graph, rootPackages, walkedDependencies);
        }
        else if (filter.startsWith('...')) {
            const leafPackagesFilter = filter.substring(3);
            const leafPackages = matchPackages(pkgGraph, leafPackagesFilter);
            if (!reversedGraph) {
                reversedGraph = reverseGraph(graph);
            }
            pickSubgraph(reversedGraph, leafPackages, walkedDependents);
        }
        else {
            Array.prototype.push.apply(cherryPickedPackages, matchPackages(pkgGraph, filter));
        }
    }
    const walked = new Set([...walkedDependencies, ...walkedDependents]);
    cherryPickedPackages.forEach((cherryPickedPackage) => walked.add(cherryPickedPackage));
    return R.pick(Array.from(walked), pkgGraph);
}
exports.filterGraph = filterGraph;
function pkgGraphToGraph(pkgGraph) {
    const graph = {};
    Object.keys(pkgGraph).forEach((nodeId) => {
        graph[nodeId] = pkgGraph[nodeId].dependencies;
    });
    return graph;
}
function reverseGraph(graph) {
    const reversedGraph = {};
    Object.keys(graph).forEach((dependentNodeId) => {
        graph[dependentNodeId].forEach((dependencyNodeId) => {
            if (!reversedGraph[dependencyNodeId]) {
                reversedGraph[dependencyNodeId] = [dependentNodeId];
            }
            else {
                reversedGraph[dependencyNodeId].push(dependentNodeId);
            }
        });
    });
    return reversedGraph;
}
function matchPackages(graph, pattern) {
    return R.keys(graph).filter((id) => graph[id].manifest.name && minimatch(graph[id].manifest.name, pattern));
}
function filterGraphByScope(graph, scope) {
    const root = matchPackages(graph, scope);
    if (!root.length)
        return {};
    const subgraphNodeIds = new Set();
    pickSubPkgGraph(graph, root, subgraphNodeIds);
    return R.pick(Array.from(subgraphNodeIds), graph);
}
exports.filterGraphByScope = filterGraphByScope;
function pickSubPkgGraph(graph, nextNodeIds, walked) {
    for (const nextNodeId of nextNodeIds) {
        if (!walked.has(nextNodeId)) {
            walked.add(nextNodeId);
            pickSubPkgGraph(graph, graph[nextNodeId].dependencies, walked);
        }
    }
}
function pickSubgraph(graph, nextNodeIds, walked) {
    for (const nextNodeId of nextNodeIds) {
        if (!walked.has(nextNodeId)) {
            walked.add(nextNodeId);
            if (graph[nextNodeId])
                pickSubgraph(graph, graph[nextNodeId], walked);
        }
    }
}
//# sourceMappingURL=filter.js.map